/******************/
/* Initialization */


/* Updates the HUE bridge IP address and registration.
 * The address is ether autodetected, or set by the optional argument.
 *
 * @param hueBridgeIPAddress optional argument specifying a manual IP address
 *        if not set, or if set to an empty string, autodetection is used
 */
function initHUEBridge(hueBridgeIPAddress)
{
	try
	{
		lInfo.logln("initHUEBridge: updating HUE bridge address information");
		updateHueBridgeAddress(hueBridgeIPAddress);
		lInfo.logln("initHUEBridge: registering this application to the HUE bridge (if not done already)");
		hueBridgeRegisterApp();

		/* If this the first time a bridge was contacted successfully,
		 * set default presets and global activities */
		var registerStatus = getHueBridgeRegisterStatus();
		if (registerStatus === HueBridgeRegisterStatus.OK)
		{
			var hueBridgeContactedBefore = Property.getProperty(hueBridgeContactedBeforePropName);
			if (!hueBridgeContactedBefore)
			{
				createStandardGlobalActivities();
				createStandardPresetsForAllZones();

				Property.setProperty(hueBridgeContactedBeforePropName, true);
				Property.setFlag(hueBridgeContactedBeforePropName, 'ARCHIVE', true);
				Property.store();
			}
		}

		checkLightsOfAllActivities();
	}
	catch (err)
	{
		lWarn.logln("initHUEBridge: Error during HUE bridge processing: " + String(err));
	}
}


/* Initializes application. */
function startup()
{
	/* First, load the property tree; this must be the very first call,
	 * since most functions in the application make use of the property tree. */
	Property.load();

	/// Check if a manual IP address has been configured already
	var manual = htmlUnescape(Property.getProperty(hueBridgeAddrManualPropName));
	lInfo.logln("startup: loading HUE bridge IP address; manual value = " + String(manual));
	if (manual)
	{
		// Manual IP address given - initialize HUE bridge with it
		var ipaddress = htmlUnescape(Property.getProperty(hueBridgeAddrPropName));
		lInfo.logln("startup: using manually configured IP address " + String(ipaddress) + " for HUE bridge");
		initHUEBridge(ipaddress);
	}
	else
	{
		// no IP address given - let the Philips Portal API find the bridge
		lInfo.logln("startup: autodetecting IP address of HUE bridge");
		initHUEBridge();
	}
}




/*********************/
/* Activity handling */


/* Undo stack explanation:
 *
 * The undo "stack" is hardcoded to contain 2 levels. Level 1 is the "current" status, level 2
 * is the "previous" one. The current status is what the user just set, the previous the one before.
 * For each status, data about the associated trigger is stored: for scene triggers, this is the scene ID,
 * for statechange triggers, this is the state name.
 *
 * When storeCurBehaviorInUndoStack() is called, the contents of level 1 is stored in level 2,
 * and level 1 is filled with the new data from the function parameters. The function does a check
 * to see if the same trigger data is already present in level 1; if so, it does not do anything.
 * This makes sure repeated triggers do not cause the stack to be updated every time.
 *
 * When handleUndo() is called, the trigger data is extracted from the function parameters, and
 * compared with the one stored in level 2. If there is a mismatch, or if level 2 is empty, then
 * this function does nothing. Otherwise, it sets the lights to whatever state is stored in level 2.
 * This check makes sure no mismatching do-undo sequences occur. Example: first, "Doorbell" is triggered,
 * then "Alarm 1", then "Undo alarm 2". In such a case, the scene IDs will not match, and the code
 * will not revert to the stored values for "Doorbell".
 */


var UndoStackNodeName = "undoStack";
var UndoStackCurrentStatusTriggerDataPropName = UndoStackNodeName + "/currentStatusTriggerData";
var UndoStackCurrentStatusBehaviorPropName = UndoStackNodeName + "/currentStatusBehavior";
var UndoStackPreviousStatusTriggerDataPropName = UndoStackNodeName + "/previousStatusTriggerData";
var UndoStackPreviousStatusBehaviorPropName = UndoStackNodeName + "/previousStatusBehavior";


function storeCurBehaviorInUndoStack(triggerData, behavior)
{
	// check if this trigger is already stored by comparing structure members
	var sameTriggerStoredAlready = false;
	// need to get the current trigger data to check
	var curTriggerDataJSON = Property.getProperty(UndoStackCurrentStatusTriggerDataPropName);
	var curTriggerData = null;
	try
	{
		if (curTriggerDataJSON !== null)
			curTriggerData = JSON.parse(htmlUnescape(curTriggerDataJSON));
	}
	catch (e)
	{
		lWarn.logln("Could not parse current trigger data in undo stack");
		return;
	}

	if (curTriggerData !== null)
	{
		sameTriggerStoredAlready = true;
		for (var v in curTriggerData)
		{
			if (triggerData[v] !== curTriggerData[v])
			{
				sameTriggerStoredAlready = false;
				break;
			}
		}
	}

	if (sameTriggerStoredAlready)
	{
		lInfo.logln("storeCurBehaviorInUndoStack: same trigger already stored -> not doing anything");
		return;
	}

	lInfo.logln("storeCurBehaviorInUndoStack: given trigger is not stored yet -> storing");

	// store current trigger data as previous one
	// (no getProperty() call necessary since the current data was retrieved earlier)
	if (curTriggerData !== null)
	{
		Property.setProperty(UndoStackPreviousStatusTriggerDataPropName, htmlUnescape(curTriggerDataJSON));
		Property.setFlag(UndoStackPreviousStatusTriggerDataPropName, 'ARCHIVE', true);
	}

	// store current behavior as previous one
	var curBehavior = Property.getProperty(UndoStackCurrentStatusBehaviorPropName);
	if (curBehavior !== null)
	{
		Property.setProperty(UndoStackPreviousStatusBehaviorPropName, htmlUnescape(curBehavior));
		Property.setFlag(UndoStackPreviousStatusBehaviorPropName, 'ARCHIVE', true);
	}

	Property.setProperty(UndoStackCurrentStatusTriggerDataPropName, JSON.stringify(triggerData));
	Property.setFlag(UndoStackCurrentStatusTriggerDataPropName, 'ARCHIVE', true);
	Property.setProperty(UndoStackCurrentStatusBehaviorPropName, JSON.stringify(behavior));
	Property.setFlag(UndoStackCurrentStatusBehaviorPropName, 'ARCHIVE', true);

	Property.store();
}


function cleanupUndo()
{
	var undoStackNode = Property.getNode(UndoStackNodeName);
	var subnode;

	subnode = Property.getNode(UndoStackPreviousStatusTriggerDataPropName);
	if (subnode)
		undoStackNode.removeChild(subnode);
	subnode = Property.getNode(UndoStackPreviousStatusBehaviorPropName);
	if (subnode)
		undoStackNode.removeChild(subnode);
}


function handleUndo(parameter)
{
	var triggerData = {};
	if (parameter.sceneID !== undefined)
	{
		triggerData["type"] = "scene";
		triggerData["sceneID"] = parseInt(parameter.sceneID);
	}
	else if (parameter.statename !== undefined)
	{
		triggerData["type"] = "statechange";
		// remove the "undo-" prefix from the name
		if (parameter.statename.substring(0, 5) == 'undo-')
			triggerData["name"] = parameter.statename.substring(5);
		else
			triggerData["name"] = parameter.statename;
	}
	else
	{
		lWarn.logln("handleUndo: unsupported trigger type");
		return;
	}

	lInfo.logln("handleUndo: trigger data: " + JSON.stringify(triggerData));

	var undoStackNode = Property.getNode(UndoStackNodeName);

	var curTriggerDataJSON = Property.getProperty(UndoStackCurrentStatusTriggerDataPropName);
	var curTriggerData = null;
	try
	{
		if (curTriggerDataJSON !== null)
			curTriggerData = JSON.parse(htmlUnescape(curTriggerDataJSON));
	}
	catch (e)
	{
		lWarn.logln("handleUndo: could not parse previous trigger data in undo stack");
		cleanupUndo(); // cleanup undo data , since it is corrupted
		return;
	}

	lInfo.logln("handleUndo: current trigger data: " + htmlUnescape(curTriggerDataJSON));

	if (curTriggerData !== null)
	{
		var triggersAreDifferent = false;
		for (var v in triggerData)
		{
			if ((triggerData[v] === undefined) || (curTriggerData[v] === undefined) || (triggerData[v] !== curTriggerData[v]))
			{
				triggersAreDifferent = true;
				break;
			}
		}

		if (triggersAreDifferent)
		{
			lInfo.logln("handleUndo: current behavior has mismatching trigger data - not doing anything");
			return;
		}
		else
			lInfo.logln("handleUndo: current behavior has matching trigger data - proceeding with undo");
	}
	else
	{
		lInfo.logln("handleUndo: no current behavior to revert to - not doing anything");
		return;
	}

	var prevBehaviorJSON = Property.getProperty(UndoStackPreviousStatusBehaviorPropName);
	var prevBehavior = null;
	if (prevBehaviorJSON !== null)
	{
		try
		{
			prevBehavior = JSON.parse(htmlUnescape(prevBehaviorJSON));
		}
		catch (err)
		{
			lWarn.logln("handleUndo: could not parse previous behavior - nothing to revert to");
			cleanupUndo(); // cleanup undo data , since it is corrupted
			return;
		}
	}

	// set previous trigger data and behavior to current one
	Property.setProperty(UndoStackCurrentStatusTriggerDataPropName, htmlUnescape(Property.getProperty(UndoStackPreviousStatusTriggerDataPropName)));
	Property.setFlag(UndoStackCurrentStatusTriggerDataPropName, 'ARCHIVE', true);
	Property.setProperty(UndoStackCurrentStatusBehaviorPropName, htmlUnescape(prevBehaviorJSON));
	Property.setFlag(UndoStackCurrentStatusBehaviorPropName, 'ARCHIVE', true);

	// delete previous trigger data and behavior
	if (undoStackNode)
	{
		var subnode;

		subnode = Property.getNode(UndoStackPreviousStatusTriggerDataPropName);
		if (subnode)
			undoStackNode.removeChild(subnode);
		subnode = Property.getNode(UndoStackPreviousStatusBehaviorPropName);
		if (subnode)
			undoStackNode.removeChild(subnode);
	}

	Property.store();

	lInfo.logln('handleUndo: reverting to light settings: ' + htmlUnescape(prevBehaviorJSON));
	sendLightsToHueBridge(prevBehavior);
}


/*var LastLightInfoPropName = "lastLightInfo";
var LastLightBehavior1PropName = LastLightInfoPropName + "/behavior1";
var LastLightBehavior2PropName = LastLightInfoPropName + "/behavior2";
var LastLightSceneID1PropName = LastLightInfoPropName + "/sceneID1";
var LastLightSceneID2PropName = LastLightInfoPropName + "/sceneID2";*/


/*function getLastLightBehavior()
{
	var behaviorProp = Property.getProperty(LastLightBehavior2PropName);
	if (!behaviorProp)
		return null;
	var behaviorJSON = htmlUnescape(behaviorProp);
	var behavior = null;
	if (behaviorJSON !== null)
	{
		try
		{
			behavior = JSON.parse(behaviorJSON);
		}
		catch (err)
		{
			lWarn.logln("getLastLightBehavior: error while parsing JSON: " + String(behaviorJSON));
		}
	}
	return behavior;
}


function storeBehaviorAsLastOne(type, params, behavior)
{
	if (!behavior)
		return;

	Property.setProperty(LastLightTriggerTypePropName, type);

	if (type === 'scene')
	{
		var sceneID = params.sceneID;
		var lastSceneID = Property.getProperty(LastLightSceneID1PropName);
		if (lastSceneID !== sceneID)
		{
			lInfo.logln('storeBehaviorAsLastOne: storing new behavior for scene ID ' + String(sceneID) + ' in the undo stack');

			var prop1Behavior = Property.getProperty(LastLightBehavior1PropName);
			if (prop1Behavior !== null)
			{
				Property.setProperty(LastLightBehavior2PropName, htmlUnescape(prop1Behavior));
				Property.setFlag(LastLightBehavior2PropName, 'ARCHIVE', true);
			}
			var prop1SceneID = Property.getProperty(LastLightSceneID1PropName);
			if (prop1SceneID !== null)
			{
				Property.setProperty(LastLightSceneID2PropName, prop1SceneID);
				Property.setFlag(LastLightSceneID2PropName, 'ARCHIVE', true);
			}

			Property.setProperty(LastLightBehavior1PropName, JSON.stringify(behavior));
			Property.setFlag(LastLightBehavior1PropName, 'ARCHIVE', true);
			Property.setProperty(LastLightSceneID1PropName, sceneID);
			Property.setFlag(LastLightSceneID1PropName, 'ARCHIVE', true);

			Property.store();
		}
		else
		{
			lInfo.logln('storeBehaviorAsLastOne: same scene ID stored already -> not storing anything');
		}
	}
	else if (type === 'statechange')
	{
		var statechangeName = params.name;
		var lastStatechangeName = Property.getProperty(LastLightStatechangeName1PropName);
		if (statechangeName !== lastStatechangeName)
		{
			lInfo.logln('storeBehaviorAsLastOne: storing new behavior for statechange ' + statechangeName + ' in the undo stack');

			var prop1Behavior = Property.getProperty(LastLightBehavior1PropName);
			if (prop1Behavior !== null)
			{
				Property.setProperty(LastLightBehavior2PropName, htmlUnescape(prop1Behavior));
				Property.setFlag(LastLightBehavior2PropName, 'ARCHIVE', true);
			}
			var prop1StatechangeName = Property.getProperty(LastLightStatechangeName1PropName);
			if (prop1StatechangeName !== null)
			{
				Property.setProperty(LastLightStatechangeName2PropName, prop1StatechangeName);
				Property.setFlag(LastLightStatechangeName2PropName, 'ARCHIVE', true);
			}

			Property.setProperty(LastLightBehavior1PropName, JSON.stringify(behavior));
			Property.setFlag(LastLightBehavior1PropName, 'ARCHIVE', true);
			Property.setProperty(LastLightStatechangeName1PropName, sceneID);
			Property.setFlag(LastLightStatechangeName1PropName, 'ARCHIVE', true);

			Property.store();
		}
		else
		{
			lInfo.logln('storeBehaviorAsLastOne: same statechange stored already -> not storing anything');
		}
	}
}


function handleUndoScene(parameter)
{
	var lastType = Property.getProperty(LastLightTriggerTypePropName);
	if (lastType !== 'scene')
	{
		lInfo.logln("handleUndoScene: last activity was of type '" + String(lastType) + "' , not of type 'scene' -> not doing anything");
		return;
	}

	if (Property.getProperty(LastLightSceneID2PropName) === null)
	{
		lInfo.logln('handleUndoScene: nothing to revert to');
		return;
	}

	var lastSceneID = Property.getProperty(LastLightSceneID1PropName);
	if (lastSceneID === null)
	{
		lInfo.logln('handleUndoScene: no last scene ID found');
		return;
	}

	var sceneID = parameter.sceneID;
	if (sceneID !== null)
	{
		if (sceneID !== lastSceneID)
		{
			lInfo.logln("handleUndoScene: given scene ID " + String(sceneID) + " does not match last scene ID " + String(lastSceneID) + " -> NOT performing an undo");
			return;
		}
	}

	var lastBehavior = getLastLightBehavior();
	if (!lastBehavior)
	{
		lInfo.logln('handleUndoScene: nothing to revert to');
		return;
	}

	Property.setProperty(LastLightBehavior1PropName, htmlUnescape(Property.getProperty(LastLightBehavior2PropName)));
	Property.setFlag(LastLightBehavior1PropName, 'ARCHIVE', true);
	Property.setProperty(LastLightSceneID1PropName, Property.getProperty(LastLightSceneID2PropName));
	Property.setFlag(LastLightSceneID1PropName, 'ARCHIVE', true);

	var prop2ParentNode = Property.getNode(LastLightInfoPropName);

	var prop2BehaviorNode = Property.getNode(LastLightBehavior2PropName);
	if (prop2ParentNode && prop2BehaviorNode)
		prop2ParentNode.removeChild(prop2BehaviorNode);

	var prop2SceneIDNode = Property.getNode(LastLightSceneID2PropName);
	if (prop2ParentNode && prop2SceneIDNode)
		prop2ParentNode.removeChild(prop2SceneIDNode);

	Property.store();

	lInfo.logln('handleUndoScene: reverting to light settings: ' + JSON.stringify(lastBehavior));
	sendLightsToHueBridge(lastBehavior);
}*/


function handleActivity(parameter)
{
	var propPath = parameter.path;

	var activated = htmlUnescape(Property.getProperty(propPath + "/activated"));
	/* NOT using an (!activated) check, since this property may not be set (this is the case for presets) */
	if (activated === false)
	{
		lInfo.logln('handleActivity: activity is deactivated');
		return;
	}

	var lightsJSON = htmlUnescape(Property.getProperty(propPath + "/lights"));

	if (lightsJSON === null)
	{
		lWarn.logln('handleActivity: found no lights field for activity at path "' + propPath + '"');
		return;
	}

	var lights = null;
	try
	{
		lights = JSON.parse(lightsJSON);
		lInfo.logln("handleActivity: sending light properties to HUE bridge: " + lightsJSON);
		sendLightsToHueBridge(lights);
	}
	catch (err)
	{
		lWarn.logln('handleActivity: error while parsing JSON data in lights field for activity at path "' + propPath + '": ' + err);
		return;
	}

	if (parameter.sceneID !== undefined)
		storeCurBehaviorInUndoStack({ type: 'scene', sceneID: parseInt(parameter.sceneID) }, lights);
	else if (parameter.statename !== undefined)
		storeCurBehaviorInUndoStack({ type: 'statechange', name: parameter.statename }, lights);
}


function handleDimming(parameter)
{
	var zoneID = parseInt(parameter.zoneID);
	var sceneID = parseInt(parameter.sceneID);

	lInfo.logln("handleDimming: zone ID " + String(zoneID) + " scene ID " + String(sceneID));

	try
	{
		switch (sceneID)
		{
			case 11: hueDimLightsInZone(zoneID, -dimmingStepSize); break;
			case 12: hueDimLightsInZone(zoneID, +dimmingStepSize); break;
			case 13: hueDimLightsInZone(zoneID, -255); break;
			case 14: hueDimLightsInZone(zoneID, +255); break;
			default: break;
		}
	}
	catch(e)
	{
		lWarn.logln('handleDimming: dimming failed: ' + String(e));
	}
}


function handleOff(parameter)
{
	var zoneID = parseInt(parameter.zoneID);
	var sceneID = parseInt(parameter.sceneID);

	lInfo.logln("handleOff: zone ID " + String(zoneID) + " scene ID " + String(sceneID));

	try
	{
		hueSwitchOffLightsInZone(zoneID);
	}
	catch(e)
	{
		lWarn.logln('handleDimming: dimming failed: ' + String(e));
	}
}




/***********************/
/* main event handling */


function parsePayload(payloadEscaped)
{
	var payload = unescape(payloadEscaped);
	try
	{
		payload = JSON.parse(payload);
	}
	catch (err)
	{
		lWarn.logln("parsePayload: JSON parse error: " + String(err));
		payload = null;
	}

	return payload;
}


// NOTE: this function will modify the contents of the payload (it will add a "msgid" member)
function respondToBackendChannelEvent(receivedEvent, payload)
{
	if (payload)
		payload.msgid = receivedEvent.parameter.msgid;
	else
		payload = { msgid: receivedEvent.parameter.msgid };

	lInfo.logln("respondToBackendChannelEvent: payload: " + JSON.stringify(payload));

	try
	{
		var responseEvent = new Event("huecontroller.backendChannel", payload);
		responseEvent.raise();
	}
	catch (err)
	{
		lWarn.logln("respondToBackendChannelEvent: error while raising response event: " + String(err));
	}
}


function returnHueBridgeInfo(raisedEvent)
{
	var info = getHueBridgeInfo();
	var infoJSON = JSON.stringify(info);
	lInfo.logln("returnHueBridgeInfo: HUE bridge info: " + infoJSON);
	var registerStatus = getHueBridgeRegisterStatus();
	var errorMsg = '';
	if (registerStatus !== HueBridgeRegisterStatus.OK)
	{
		errorMsg = Property.getProperty(hueRegisterErrorMsgPropName);
		if (errorMsg === null)
			errorMsg = '';
	}

	// send response event
	respondToBackendChannelEvent(raisedEvent, { info: infoJSON, registerStatus: registerStatus, errorMsg: errorMsg });
}


// TODO: comment this out in release builds
lInfo.logln("eventhandling: huecontroller event raised: " + JSON.stringify(raisedEvent));

try
{
	if (raisedEvent.name == 'hueController.handleTrigger')
	{
		handleActivity(raisedEvent.parameter);
	}
	else if (raisedEvent.name == 'hueController.handleDimming')
	{
		handleDimming(raisedEvent.parameter);
	}
	else if (raisedEvent.name == 'hueController.handleOff')
	{
		handleOff(raisedEvent.parameter);
	}
	else if (raisedEvent.name == 'stateChange')
	{
		if (raisedEvent.parameter.state === 'inactive')
			handleUndo(raisedEvent.parameter);
	}
	else if (raisedEvent.name == 'undoScene')
	{
		handleUndo(raisedEvent.parameter);
	}
	else if (raisedEvent.name == 'running')
	{
		startup();
	}
	else if (raisedEvent.name == 'blinkingTimeout')
	{
		var blinkID = Property.getProperty(hueBlinkingIDPropName);
		lInfo.logln("eventhandling: blinking timeout, stored blinkID: " + String(blinkID) + " event blinkID: " + String(raisedEvent.parameter.blinkID));
		if (blinkID === parseInt(raisedEvent.parameter.blinkID))
		{
			lInfo.logln("eventhandling: timeout valid - setting lights to old value");
			var oldLightAttributesJSON = htmlUnescape(Property.getProperty(hueBlinkingAttributesPropName));

			try
			{
				var oldLightAttributes = JSON.parse(oldLightAttributesJSON);
				handleBlinkingTimeout(oldLightAttributes);
			}
			catch (e)
			{
				lWarn.logln("eventhandling: error while resetting lights after blinking: " + String(e));
			}

			Property.setProperty(hueBlinkingIDPropName, 0);
		}
		else
			lInfo.logln("eventhandling: timeout no longer valid - ignoring");
	}
	else if (raisedEvent.name == 'huecontroller')
	{
		/* activity events */
		if (raisedEvent.parameter.action == 'deleteActivity')
		{
			var payload = parsePayload(raisedEvent.parameter.payload);
			if (payload)
			{
				lInfo.logln('eventhandling: deleting activity of type "' + payload.activityType + '" with ID ' + String(payload.activityID));
				deleteActivity(payload.activityID, payload.activityType);
			}

			respondToBackendChannelEvent(raisedEvent);
		}
		else if (raisedEvent.parameter.action == 'testActivity')
		{
			var payload = parsePayload(raisedEvent.parameter.payload);
			if (payload)
			{
				lInfo.logln('eventhandling: test-running activity of type "' + payload.activityType + '" with ID ' + String(payload.activityID));
				testActivity(payload.activityID, payload.activityType);
			}
		}
		else if (raisedEvent.parameter.action == 'setActivity')
		{
			var payload = parsePayload(raisedEvent.parameter.payload);
			if (payload)
			{
				if (payload.activityID === null)
					lInfo.logln('eventhandling: setting activity of type "' + payload.activityType + '" (new ID will be created)');
				else
					lInfo.logln('eventhandling: setting activity of type "' + payload.activityType + '" with ID ' + String(payload.activityID));
				lInfo.logln('eventhandling: payload = ' + JSON.stringify(payload)); // TODO: comment out in release builds
				setActivity(payload.activityID, payload.activityType, payload.activity);
			}

			respondToBackendChannelEvent(raisedEvent);
		}
		else if (raisedEvent.parameter.action == 'getActivity')
		{
			var payload = parsePayload(raisedEvent.parameter.payload);
			if (payload)
			{
				lInfo.logln('eventhandling: getting activity of type "' + payload.activityType + '" with ID ' + String(payload.activityID));

				var activity = getActivity(payload.activityID, payload.activityType);
				if (activity)
				{
					// send response event
					respondToBackendChannelEvent(raisedEvent, { activityID: payload.activityID, activityType: payload.activityType, activity: JSON.stringify(activity) });
				}
			}
		}
		else if (raisedEvent.parameter.action == 'getAllActivities')
		{
			var payload = parsePayload(raisedEvent.parameter.payload);
			if (payload)
			{
				lInfo.logln('eventhandling: returning all activities to frontend');
				var activities = getAllActivities(payload.activityType);
				var activitiesJSON = JSON.stringify(activities);

				// send response event
				respondToBackendChannelEvent(raisedEvent, { activityType: payload.activityType, activities: activitiesJSON });
			}
		}
		/* HUE bridge events */
		else if (raisedEvent.parameter.action == 'setHueBridgeAddress')
		{
			var payload = parsePayload(raisedEvent.parameter.payload);
			if (payload)
			{
				var ipaddress = payload.ipaddress;
				if (ipaddress)
					lInfo.logln("eventhandling: setHueBridgeAddress called with manual IP address " + JSON.stringify(ipaddress));
				else
					lInfo.logln("eventhandling: setHueBridgeAddress called with autodetection requested");
				initHUEBridge(ipaddress);

				returnHueBridgeInfo(raisedEvent);
			}
		}
		else if (raisedEvent.parameter.action == 'findNewLights')
		{
			lInfo.logln("eventhandling: instructing HUE bridge to search for new lights");
			hueFindNewLights();
		}
		else if (raisedEvent.parameter.action == 'testLight')
		{
			var payload = parsePayload(raisedEvent.parameter.payload);
			if (payload)
			{
				lInfo.logln('eventhandling: setting test values for light with ID ' + String(payload.lightID));
				testLight(payload);
			}
		}
		else if (raisedEvent.parameter.action == 'getHueBridgeInfo')
		{
			lInfo.logln("eventhandling: getting HUE bridge information");
			returnHueBridgeInfo(raisedEvent);
		}
		else if (raisedEvent.parameter.action == 'registerHueBridge')
		{
			lInfo.logln("eventhandling: registering this application to the HUE bridge (if not done already)");
			hueBridgeRegisterApp();

			returnHueBridgeInfo(raisedEvent);
		}
		else if (raisedEvent.parameter.action == 'setHueLightZoneAssociation')
		{
			var payload = parsePayload(raisedEvent.parameter.payload);
			if (payload)
			{
				lInfo.logln("eventhandling: updating light<->zone association: " + JSON.stringify(payload));
				setHueLightZoneAssociation(payload.lightID, payload.zoneID);

				returnHueBridgeInfo(raisedEvent);
			}
		}
		else if (raisedEvent.parameter.action == 'getHueBridgeLightAttributes')
		{
			var payload = parsePayload(raisedEvent.parameter.payload);
			if (payload)
			{
				lInfo.logln("eventhandling: getting HUE light attributes for light of ID " + String(payload.lightID));
				var attributes = getHueBridgeLightAttributes(payload.lightID);
				var attributesJSON = JSON.stringify(attributes);
				lInfo.logln("eventhandling: light attributes: " + attributesJSON);

				// send response event

				// need to transmit attributes as JSON, because the event itself
				// cannot support too complex values (for example, objects containing other objects)
				respondToBackendChannelEvent(raisedEvent, { attributes: attributesJSON });
			}
		}
	}
}
catch (err)
{
	lWarn.logln("[ERROR] eventhandling: " + getExceptionObjectDetails(err));
}

